// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
// Wait for Cordova to load
//

angular.module('comprecar', [
    'ionic',
    'ngCordova',
    'ui.ace',
    'http-auth-interceptor',
    'comprecar.menu.ctrl',
    'comprecar.search.ctrl',
    'comprecar.result.ctrl',
    'comprecar.details.ctrl',
    'comprecar.cidade.ctrl',
    'comprecar.ano.ctrl',
    'comprecar.price.ctrl',
    'comprecar.tipo.ctrl',
    'comprecar.ads.ctrl',
    'comprecar.addads.ctrl',
    'comprecar.profile.ctrl',
    'comprecar.register.ctrl',
    'comprecar.bookmark.ctrl',
    'comprecar.history.ctrl',
    'comprecar.calendar.ctrl',
    'comprecar.stores.ctrl',
    'comprecar.store.ctrl',
    'comprecar.event.ctrl',
    'comprecar.cars.srv',
    'comprecar.calendar.srv',
    'comprecar.cities.srv',
    'comprecar.states.srv',
    'comprecar.auth.srv',
    'comprecar.user.srv',
    'comprecar.brands.srv',
    'comprecar.fuel.srv',
    'comprecar.accessories.srv',
    'comprecar.stats.srv',
    'comprecar.stores.srv',
    'comprecar.contact.srv',
    'comprecar.contact.modal',
    'comprecar.vendedor.ctrl',
    'comprecar.magazine.srv',
    'comprecar.magazine.ctrl'
])
        .constant("CONSTANTS", {
            PROD: 1,
            HOST: (1) ? 'http://apicomprecar.absoluto.mobi/api/v1' : 'http://apicomprecar.local/api/v1',
            GA: (1) ? 'UA-59852472-1' : 'UA-60255229-1'
        })

        .run(function ($ionicPlatform, $httpBackend, $http, $localstorage, AuthenticationService, CONSTANTS) {

            $ionicPlatform.ready(function () {

                // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
                // for form inputs)
                if (window.cordova && window.cordova.plugins.Keyboard) {
                    //cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
                    //cordova.plugins.Keyboard.disableScroll(true);
                    cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
                }
                if (window.StatusBar) {
                    StatusBar.styleLightContent();
                }
                window.plugins.orientationLock.unlock();
                
                if (typeof window.analytics !== "undefined") {
                    window.analytics.startTrackerWithId(CONSTANTS.GA);
                    
                    alert(window.analytics);
                }               
            });
            AuthenticationService.init();
        })



        .factory('$string', ['$parse', function ($parse) {
                return {
                    serialize: function (obj, encode) {
                        var str = [];
                        for (var p in obj) {
                            if (obj.hasOwnProperty(p)) {
                                if (encode === false) {
                                    str.push((p) + "=" + (obj[p]));
                                } else {
                                    str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
                                }

                            }
                        }
                        return str.join("&");
                    }
                };
            }])
        .factory('$localstorage', ['$window', function ($window) {
                return {
                    clear: function () {
                        $window.localStorage.clear();
                    },
                    set: function (key, value) {
                        $window.localStorage[key] = value;
                    },
                    get: function (key, defaultValue) {
                        return $window.localStorage[key] || defaultValue;
                    },
                    setObject: function (key, value) {
                        $window.localStorage[key] = JSON.stringify(value);
                    },
                    getObject: function (key) {

                        try {
                            return JSON.parse($window.localStorage[key] || '[]');

                        } catch (e) {
                            $window.localStorage[key] = [];
                            return JSON.parse($window.localStorage[key] || '[]');
                        }

                    }
                }
            }])
        .directive('selectOnClick', function () {
            return {
                restrict: 'A',
                link: function (scope, element, attrs) {
                    element.on('click', function () {
                        alert(this.value());
                        this.select();
                    });
                }
            };
        })
        .directive('selectable', [function () {
                return {
                    restrict: 'A',
                    priority: 2000,
                    link: function (scope, ele, attrs) {

                        var element = ele[0];
                        function leave() {
                            element.blur();
                            element.setAttribute('contenteditable', 'false');
                        }
                        function keydown(e) {
                            alert(attrs);
                            switch (e.which) {
                                case 33: // pageup
                                case 34: // pagedown
                                case 35: // end
                                case 36: // home
                                case 37: // left
                                case 38: // up
                                case 39: // right
                                case 40: // down
                                case 17: // ctrl
                                case 91: // meta
                                    return;

                                default:
                                    //CTRL-A /CTRL-C?
                                    if ((e.keyCode === 'C'.charCodeAt(0) || e.keyCode === 'A'.charCodeAt(0)) && (e.ctrlKey || e.metaKey)) {
                                        return;
                                    }
                                    break;
                            }

                            leave();
                        }
                        function mouseDown() {
                            element.setAttribute('contenteditable', 'true');
                        }
                        element.addEventListener('mousedown', mouseDown);
                        element.addEventListener('keydown', keydown);
                        element.addEventListener('cut', leave);
                        element.addEventListener('paste', leave);
                        ele.on('$destroy', function () {
                            element.removeEventListener('mousedown', mouseDown);
                            element.removeEventListener('keydown', keydown);
                            element.removeEventListener('cut', leave);
                            element.removeEventListener('paste', leave);
                        });

                    }
                };
            }])




        .config(function ($stateProvider, $urlRouterProvider, $httpProvider, $locationProvider, CONSTANTS) {

            $stateProvider

                    .state('app', {
                        url: "/app",
                        abstract: true,
                        templateUrl: "app/menu/menu.html",
                        controller: 'MenuController'
                    })

                    .state('app.search', {
                        url: "/search",
                        views: {
                            'menuContent': {
                                templateUrl: "app/search/search.html",
                                controller: 'SearchController'
                            }
                        }
                    })
                    .state('app.details', {
                        url: "/details/:id",
                        views: {
                            'menuContent': {
                                templateUrl: "app/details/details.html",
                                controller: 'DetailsController'
                            }
                        }
                    })

                    .state('app.event', {
                        url: "/event",
                        views: {
                            'menuContent': {
                                templateUrl: "app/calendar/event.html",
                                controller: 'EventController'
                            }
                        }
                    })

                    .state('app.result', {
                        url: "/results",
                        views: {
                            'menuContent': {
                                templateUrl: "app/result/result.html",
                                controller: 'ResultController'
                            }
                        }
                    })
                    .state('app.bookmark', {
                        url: "/bookmark",
                        views: {
                            'menuContent': {
                                templateUrl: "app/bookmark/bookmark.html",
                                controller: 'BookmarkController'
                            }
                        }
                    })



                    .state('app.profile', {
                        url: "/profile",
                        views: {
                            'menuContent': {
                                templateUrl: "app/account/profile.html",
                                controller: 'ProfileController'
                            }
                        }
                    })
                    .state('app.register', {
                        url: "/register",
                        views: {
                            'menuContent': {
                                templateUrl: "app/account/register.html",
                                controller: 'RegisterController'
                            }
                        }
                    })
                    .state('app.addads', {
                        url: "/addads",
                        views: {
                            'menuContent': {
                                templateUrl: "app/ads/addads.html",
                                controller: 'AddadsController'
                            }
                        }
                    })
                    .state('app.ads', {
                        url: "/ads",
                        views: {
                            'menuContent': {
                                templateUrl: "app/ads/ads.html",
                                controller: 'AdsController'
                            }
                        }
                    })
                    .state('app.history', {
                        url: "/history",
                        views: {
                            'menuContent': {
                                templateUrl: "app/history/history.html",
                                controller: 'HistoryController'
                            }
                        }
                    })
                    .state('app.tipo', {
                        url: "/tipo/:type",
                        views: {
                            'menuContent': {
                                templateUrl: "app/tipo/tipo.html",
                                controller: 'TipoController'
                            }
                        }
                    })

                    .state('app.stores', {
                        url: "/stores",
                        views: {
                            'menuContent': {
                                templateUrl: "app/stores/stores.html",
                                controller: 'StoresController'
                            }
                        }
                    })

                    .state('app.magazine', {
                        url: "/magazine",
                        views: {
                            'menuContent': {
                                templateUrl: "app/magazine/magazine.html",
                                controller: 'MagazineController'
                            }
                        }
                    })
                    .state('app.store', {
                        url: "/store/:id",
                        views: {
                            'menuContent': {
                                templateUrl: "app/store/store.html",
                                controller: 'StoreController'
                            }
                        }
                    })

                    .state('app.vendedor', {
                        url: "/vendedor/:customer_type",
                        views: {
                            'menuContent': {
                                templateUrl: "app/vendedor/vendedor.html",
                                controller: 'VendedorController'
                            }
                        }
                    })
                    .state('app.price', {
                        url: "/price/",
                        views: {
                            'menuContent': {
                                templateUrl: "app/price/price.html",
                                controller: 'PriceController'
                            }
                        }
                    })
                    .state('app.cidade', {
                        url: "/cidade/:city_id",
                        views: {
                            'menuContent': {
                                templateUrl: "app/cidade/cidade.html",
                                controller: 'CidadeController'
                            }
                        }
                    })
                    .state('app.ano', {
                        url: "/ano/",
                        views: {
                            'menuContent': {
                                templateUrl: "app/ano/ano.html",
                                controller: 'AnoController'
                            }
                        }
                    })
                    .state('app.calendar', {
                        url: "/calendar",
                        views: {
                            'menuContent': {
                                templateUrl: "app/calendar/calendar.html",
                                controller: 'CalendarController'
                            }
                        }
                    })


                    ;
            // if none of the above states are matched, use this as the fallback
            $urlRouterProvider.otherwise('/app/search');
        });



