
angular.module('comprecar.calendar.ctrl', [])
        .controller('CalendarController', ['$scope', '$ionicLoading', '$rootScope', '$state', '$ionicViewService', '$ionicModal', '$cordovaDialogs', '$ionicNavBarDelegate', '$cordovaDatePicker', '$filter', 'CalendarSrc', function ($scope, $ionicLoading, $rootScope, $state, $ionicViewService, $ionicModal, $cordovaDialogs, $ionicNavBarDelegate, $cordovaDatePicker, $filter, CalendarSrc) {

                moment.locale('pt-br');
                var today = moment();

                function Calendar(selector, events) {
                    this.el = document.querySelector(selector);
                    this.events = events;
                    this.current = moment().date(1);

                    this.draw();
                    var current = document.querySelector('.today');
                    if (current) {
                        var self = this;
                        window.setTimeout(function () {
                            self.openDay(current);
                        }, 500);
                    }
                }


                Calendar.prototype.draw = function () {
                    //Create Header
                    this.drawHeader();

                    //Draw Month
                    this.drawMonth();

                }

                Calendar.prototype.drawHeader = function () {
                    var self = this;
                    if (!this.header) {
                        //Create the header elements
                        this.header = createElement('div', 'header bar');
                        this.title = createElement('h1', 'h1 title');


                        var left = createElement('button', 'button button-icon icon ion-ios7-arrow-back');
                        left.addEventListener('click', function () {
                            self.prevMonth();
                        });
                        var right = createElement('button', 'button button-icon icon ion-ios7-arrow-forward');
                        right.addEventListener('click', function () {
                            self.nextMonth();
                        });



                        //Append the Elements
                        this.header.appendChild(this.title);
                        this.header.appendChild(left);
                        this.header.appendChild(right);
                        this.el.appendChild(this.header);
                    }

                    this.title.innerHTML = this.current.format('MMMM YYYY');
                }

                Calendar.prototype.drawMonth = function () {
                    var self = this;
                    this.events.forEach(function (ev) {
                        ev.date = moment(ev.date_due, "YYYY-MM-DD HH:mm");
                    });


                    if (this.month) {
                        this.removeLegend();
                        this.oldMonth = this.month;
                        this.oldMonth.className = 'month out ' + (self.next ? 'next' : 'prev');
                        this.oldMonth.addEventListener('webkitAnimationEnd', function () {
                            self.oldMonth.parentNode.removeChild(self.oldMonth);
                            self.month = createElement('div', 'month');
                            self.backFill();
                            self.currentMonth();
                            self.fowardFill();
                            self.el.appendChild(self.month);
                            window.setTimeout(function () {
                                self.month.className = 'month in ' + (self.next ? 'next' : 'prev');
                                window.setTimeout(function () {
                                    self.drawLegend();
                                }, 20);
                            }, 16);
                        });

                    } else {
                        this.month = createElement('div', 'month');
                        this.el.appendChild(this.month);
                        this.backFill();
                        this.currentMonth();
                        this.fowardFill();
                        this.month.className = 'month new';
                        window.setTimeout(function () {
                            self.drawLegend();
                        }, 20);

                    }

                }

                Calendar.prototype.backFill = function () {
                    var clone = this.current.clone();
                    var dayOfWeek = clone.day();

                    if (!dayOfWeek) {
                        return;
                    }

                    clone.subtract(dayOfWeek + 1, 'days');

                    for (var i = dayOfWeek; i > 0; i--) {
                        this.drawDay(clone.add(1, 'days'));
                    }
                }

                Calendar.prototype.fowardFill = function () {
                    var clone = this.current.clone().add(1, 'months').subtract(1, 'days');
                    var dayOfWeek = clone.day();

                    if (dayOfWeek === 6) {
                        return;
                    }

                    for (var i = dayOfWeek; i < 6; i++) {
                        this.drawDay(clone.add(1, 'days'));
                    }
                };

                Calendar.prototype.currentMonth = function () {
                    var clone = this.current.clone();

                    while (clone.month() === this.current.month()) {
                        this.drawDay(clone);
                        clone.add(1, 'days');
                    }
                };

                Calendar.prototype.getWeek = function (day) {
                    if (!this.week || day.day() === 0) {
                        this.week = createElement('div', 'week');
                        this.month.appendChild(this.week);
                    }
                }

                Calendar.prototype.drawDay = function (day) {
                    var self = this;
                    this.getWeek(day);

                    //Outer Day
                    var outer = createElement('div', this.getDayClass(day));
                    outer.addEventListener('click', function () {
                        self.openDay(this);
                    });

                    //Day Name
                    var name = createElement('div', 'day-name', day.format('ddd'));

                    //Day Number
                    var number = createElement('div', 'day-number', day.format('DD'));


                    //Events
                    var events = createElement('div', 'day-events');
                    this.drawEvents(day, events);

                    outer.appendChild(name);
                    outer.appendChild(number);
                    outer.appendChild(events);
                    this.week.appendChild(outer);
                }

                Calendar.prototype.drawEvents = function (day, element) {
                    if (day.month() === this.current.month()) {
                        var todaysEvents = this.events.reduce(function (memo, ev) {
                            if (ev.date.isSame(day, 'day')) {
                                memo.push(ev);
                            }
                            return memo;
                        }, []);

                        todaysEvents.forEach(function (ev) {
                            var evSpan = createElement('span', ev.color);
                            element.appendChild(evSpan);
                        });
                    }
                }

                Calendar.prototype.getDayClass = function (day) {
                    classes = ['day'];
                    if (day.month() !== this.current.month()) {
                        classes.push('other');
                    } else if (today.isSame(day, 'day')) {
                        classes.push('today');
                    }
                    return classes.join(' ');
                }

                Calendar.prototype.openDay = function (el) {
                    var details, arrow;
                    var dayNumber = +el.querySelectorAll('.day-number')[0].innerText || +el.querySelectorAll('.day-number')[0].textContent;
                    var day = this.current.clone().date(dayNumber);

                    var currentOpened = document.querySelector('.details');

                    //Check to see if there is an open detais box on the current row
                    if (currentOpened && currentOpened.parentNode === el.parentNode) {
                        details = currentOpened;
                        arrow = document.querySelector('.arrow');
                    } else {
                        //Close the open events on differnt week row
                        //currentOpened && currentOpened.parentNode.removeChild(currentOpened);
                        if (currentOpened) {
                            currentOpened.addEventListener('webkitAnimationEnd', function () {
                                currentOpened.parentNode.removeChild(currentOpened);
                            });
                            currentOpened.addEventListener('oanimationend', function () {
                                currentOpened.parentNode.removeChild(currentOpened);
                            });
                            currentOpened.addEventListener('msAnimationEnd', function () {
                                currentOpened.parentNode.removeChild(currentOpened);
                            });
                            currentOpened.addEventListener('animationend', function () {
                                currentOpened.parentNode.removeChild(currentOpened);
                            });
                            currentOpened.className = 'details out';
                        }

                        //Create the Details Container
                        details = createElement('div', 'details in');

                        //Create the arrow
                        var arrow = createElement('div', 'arrow');

                        //Create the event wrapper

                        details.appendChild(arrow);
                        el.parentNode.appendChild(details);
                    }

                    var todaysEvents = this.events.reduce(function (memo, ev) {
                        if (ev.date.isSame(day, 'day')) {
                            memo.push(ev);
                        }
                        return memo;
                    }, []);

                    this.renderEvents(todaysEvents, details);

                    arrow.style.left = el.offsetLeft - el.parentNode.offsetLeft + 27 + 'px';
                }

                Calendar.prototype.renderEvents = function (events, ele) {
                    //Remove any events in the current details element
                    var currentWrapper = ele.querySelector('.events');
                    var wrapper = createElement('div', 'list events in' + (currentWrapper ? ' new' : ''));

                    events.forEach(function (ev) {
                        var div = createElement('a', 'event item item-icon-left item-icon-right', '', ev.calendar_id);
                        var square = createElement('div', 'event-category ' + ev.color);
                        var span = createElement('span', 'open_' + ev.calendar_id, ev.subject);
                        var arrow = createElement('i', 'icon ion-ios7-arrow-right arrow-event');

                        //<i class="icon ion-arrow-right-c"></i>
                        div.appendChild(square);
                        div.appendChild(span);
                        div.appendChild(arrow);
                        wrapper.appendChild(div);
                    });

                    if (!events.length) {
                        var div = createElement('div', 'event empty');
                        var span = createElement('span', '', 'Nenhum evento');

                        div.appendChild(span);
                        wrapper.appendChild(div);
                    }

                    if (currentWrapper) {
                        currentWrapper.className = 'events out';
                        currentWrapper.addEventListener('webkitAnimationEnd', function () {
                            currentWrapper.parentNode.removeChild(currentWrapper);
                            ele.appendChild(wrapper);
                        });
                        currentWrapper.addEventListener('oanimationend', function () {
                            currentWrapper.parentNode.removeChild(currentWrapper);
                            ele.appendChild(wrapper);
                        });
                        currentWrapper.addEventListener('msAnimationEnd', function () {
                            currentWrapper.parentNode.removeChild(currentWrapper);
                            ele.appendChild(wrapper);
                        });
                        currentWrapper.addEventListener('animationend', function () {
                            currentWrapper.parentNode.removeChild(currentWrapper);
                            ele.appendChild(wrapper);
                        });
                    } else {
                        ele.appendChild(wrapper);
                    }
                }

                Calendar.prototype.removeLegend = function () {
                    var legend = document.querySelector('.legend');

                    if (legend) {
                        legend.parentElement.removeChild(legend);
                    }
                }

                Calendar.prototype.drawLegend = function () {

                    var legend = createElement('div', 'legend');
                    var calendars = this.events.map(function (e) {
                        return e.subject + '|' + e.color;
                    }).reduce(function (memo, e) {
                        if (memo.indexOf(e) === -1) {
                            memo.push(e);
                        }
                        return memo;
                    }, []).forEach(function (e) {
                        var parts = e.split('|');
                        var entry = createElement('span', 'entry ' + parts[1], parts[0]);
                        legend.appendChild(entry);
                    });
                    this.el.appendChild(legend);
                }

                Calendar.prototype.nextMonth = function () {
                    this.current.add(1, 'months');
                    this.next = true;
                    this.draw();
                }

                Calendar.prototype.prevMonth = function () {
                    this.current.subtract(1, 'months');
                    this.next = false;
                    this.draw();
                }

                window.Calendar = Calendar;

                function createElement(tagName, className, innerText, onclick) {
                    var ele = document.createElement(tagName);
                    if (className) {
                        ele.className = className;
                    }
                    if (innerText) {
                        ele.innderText = ele.textContent = innerText;
                    }
                    if (onclick) {
                        ele.addEventListener('click', function () {
                            $scope.openEvent(onclick);
                        });
                    }
                    return ele;
                }
                $scope.items = [];


                $scope.openEvent = function (id) {
                    $rootScope.eventData = $scope.items[id];
                    $state.go('app.event');
                };
                $ionicLoading.show({
                    template: '<i class="icon ion-loading-c"></i><br />',
                    animation: 'fade-in',
                    showBackdrop: true
                });

                CalendarSrc.getAll()
                        .success(function (data) {

                            $ionicLoading.hide();
                            if (data.status == 1) {
                                // Push all of the commits from response into model.
                                angular.forEach(data.result, function (item) {
                                    $scope.items[item.calendar_id] = item;
                                });
                            } else {
                                $scope.items = [];
                            }
                        })
                        .error(function (data, status, headers, config) {
                            var msg = (data.message != "") ? data.message : 'Verifique sua conexão de dados';
//                            $cordovaDialogs.alert(msg, 'Atenção ', 'Voltar')
//                                    .then(function () {
//                                        
//                                    });
                            $ionicLoading.hide();
                        }).finally(function (data, status, headers, config) {
                            
                            var calendar = new Calendar('#calendar', $scope.items);
                            $ionicLoading.hide();
                        });


                $scope.dataEventAdd = {};
                $scope.dataEvent = {};
                $scope.dataEvent.date = moment().format("YYYY-MM-DD");
                $scope.dataEvent.time = moment().format("HH:mm");

                // Create the login modal that we will use later
                $ionicModal.fromTemplateUrl('app/calendar/add.html', {
                    scope: $scope
                }).then(function (modal) {
                    $scope.modalEvent = modal;
                });
                // Triggered in the login modal to close it
                $scope.closeAddEvent = function () {
                    $scope.modalEvent.hide();
                };
                $scope.addEvent = function () {
                    $scope.modalEvent.show();
                };
                // Perform the login action when the user submits the login form
                $scope.doAddEvent = function () {
                    // Form data for the login modal
                    $scope.dataEvent.date_due = $scope.dataEvent.date + ' ' + $scope.dataEvent.time;

                    $ionicLoading.show({
                        template: '<i class="icon ion-loading-c"></i><br />Aguarde',
                        animation: 'fade-in',
                        showBackdrop: true
                    });
                    CalendarSrc.create($scope.dataEvent)
                            .success(function (data) {
                                $ionicLoading.hide();
                                if (data.status == 1) {
                                    $scope.closeAddEvent();
                                    $state.go('app.calendar', {}, {reload: true, inherit: false});
                                } else {
                                    var msg = (data.message) ? data.message : 'Verifique sua conexão de dados';
                                    $cordovaDialogs.alert(msg, 'Evento ', 'Ok');
                                }
                            })
                            .error(function (data, status, headers, config) {
                                $ionicLoading.hide();
                                var msg = (data.message) ? data.message : 'Verifique sua conexão de dados';
                                $cordovaDialogs.alert(msg, 'Evento ', 'Voltar')
                                        .then(function () {


                                        });
                            });
                };


            }]);

